@@ -9,6 +9,7 @@
 #' @param link A link function to apply to the linear predictor. 
 #'             By default, it is the identity function \code{function(x)\{x\}}, 
 #'             but it can be any function such as \code{plogis} for logistic regression models.
+#' @param x_train Training design matrix to be provided when imputations are to be made from them
 #' @param ...  Additional arguments to pass to prediction function.
 #'
 #' @return A numeric vector of predicted values for the given data \code{x}. 
@@ -34,12 +35,49 @@
 #' }
 #'
 #' @export
-predict.bgnlm_model <- function(object, x, link = function(x) { x }, ... ) {
-  x.precalc <- model.matrix(
-    as.formula(paste0("~I(", paste0(names(object$coefs)[-1], collapse = ")+I("), ")")),
-    data = x
-  )
-  yhat <- link(x.precalc %*% object$coefs)
+predict.bgnlm_model <- function(object, x, link = function(x) {x}, x_train = NULL, ... ) {
+  if(is.null(x_train))
+    x <- impute_x(object, x)
+  else
+    x <- impute_x_pred(object, x_test = x, x_train = x_train)
+  x <- data.frame(x)
+  if (object$needs.precalc) {
+    if (object$intercept) {
+      x <- cbind(1, x)
+    }
+    
+    if(length(object$features)==0)
+    {
+      warning("MPM has no featres included! All posteriors below 0.5! Baseline only used.")
+      x.precalc <-  model.matrix(~1, data = x)
+    } else precalc <- precalc.features(list(x = x, y = NULL, fixed = object$fixed), object$features)
+    
+    if (dim(precalc$x)[2]>length(object$coefs[object$coefs!=0])) {
+      precalc$x <- as.matrix(precalc$x[,-1])
+    }
+    
+    yhat <- link(precalc$x %*% object$coefs[object$coefs != 0])
+  } else {
+    
+    if(length(object$coefs)==1)
+    {
+      warning("MPM has no featres included! All posteriors below 0.5! Baseline only used.")
+      x.precalc <-  model.matrix(~1, data = x)
+    }
+    else{
+      x.precalc <- model.matrix(
+        as.formula(paste0("~I(", paste0(names(object$coefs)[-1][object$coefs[-1]!=0], collapse = ")+I("), ")")),
+        data = x
+      )
+    }
+    
+    if (dim(x.precalc)[2]<length(object$coefs[object$coefs!=0])) {
+      x.precalc <- cbind(1,x.precalc)
+    } else if (dim(x.precalc)[2]>length(object$coefs[object$coefs!=0])) {
+      x.precalc <- as.matrix(x.precalc[,-1])
+    }
+    yhat <- link(x.precalc %*% object$coefs[object$coefs!=0])
+  }
   return(yhat)
 }
 
@@ -53,37 +91,23 @@ predict.bgnlm_model <- function(object, x, link = function(x) { x }, ... ) {
 #' 
 #' @examples
 #' result <- gmjmcmc(
-#'  matrix(rnorm(600), 100),
+#'  x = matrix(rnorm(600), 100),
+#'  y = matrix(rnorm(100), 100),
 #'  P = 2,
-#'  gaussian.loglik,
-#'  loglik.alpha = gaussian.loglik.alpha,
-#'  c("p0", "exp_dbl")
+#'  transforms = c("p0", "exp_dbl")
 #' )
 #' preds <- predict(result, matrix(rnorm(600), 100))
 #' 
 #' 
 #' @export
-predict.gmjmcmc <- function (object, x, link = function(x) x, quantiles = c(0.025, 0.5, 0.975),  pop = NULL,tol =  0.0000001, ...) {
+predict.gmjmcmc <- function (object, x, link = function(x) x, quantiles = c(0.025, 0.5, 0.975),  pop = NULL,tol =  0.0000001, x_train = NULL, ...) {
   transforms.bak <- set.transforms(object$transforms)
+  if(is.null(x_train))
+    x <- impute_x(object, x)
+  else
+    x <- impute_x_pred(object, x, x_train)
   
-  
-  
-  if(!is.null(attr(object,which = "imputed")))
-  {
-    df <- data.frame(x)
-    na.matr <- data.frame(1*(is.na(df)))
-    cm <- colMeans(na.matr)
-    na.matr <- na.matr[,attr(object,which = "imputed")]
-    names(na.matr) <- paste0("mis_",names(na.matr))
-    for (i in which(cm!=0)){
-      df[[i]][is.na(df[[i]])] <- median(df[[i]], na.rm = TRUE)
-    }
-    x <- as.matrix(data.frame(df,na.matr))
-    rm(df)
-    rm(na.matr)
-  } else x <- as.matrix(x)
-  
-  merged <- merge_results(list(object),data = cbind(1, x), populations = pop, tol = tol)
+  merged <- merge_results(list(object), data = list(x = x, object$fixed), populations = pop, tol = tol)
   set.transforms(transforms.bak)
   return(predict.gmjmcmc_merged(merged, x, link, quantiles))
 }
@@ -93,26 +117,15 @@ predict.gmjmcmc <- function (object, x, link = function(x) x, quantiles = c(0.02
 #' @inheritParams predict.gmjmcmc_merged
 #' @param pop The population to use.
 #' @noRd
-predict.gmjmcmc.2 <- function (object, x, link = function(x) x, quantiles = c(0.025, 0.5, 0.975), pop = 1, ...) {
+predict.gmjmcmc.2 <- function (object, x, link = function(x) x, quantiles = c(0.025, 0.5, 0.975), pop = 1, x_train = NULL, ...) {
   transforms.bak <- set.transforms(object$transforms)
-  
-  if(!is.null(attr(object,which = "imputed")))
-  {
-    df <- data.frame(x)
-    na.matr <- data.frame(1*(is.na(df)))
-    cm <- colMeans(na.matr)
-    na.matr <- na.matr[,attr(object,which = "imputed")]
-    names(na.matr) <- paste0("mis_",names(na.matr))
-    for (i in which(cm!=0)){
-      df[[i]][is.na(df[[i]])] <- median(df[[i]], na.rm = TRUE)
-    }
-    x <- as.matrix(data.frame(df,na.matr))
-    rm(df)
-    rm(na.matr)
-  } else x <- as.matrix(x)
+  if(is.null(x_train))
+    x <- impute_x(object, x)
+  else
+    x <- impute_x_pred(object, x, x_train)
   
   mmodel <- lapply(object[1:8], function (x) x[[pop]])
-
+  
   # Precalculate the features for the new data (c(0,1...) is because precalc features thinks there is an intercept and y col).
   x.precalc <- precalc.features(cbind(0, 1, x), mmodel$populations)[, -1]
   set.transforms(transforms.bak)
@@ -127,6 +140,7 @@ predict.gmjmcmc.2 <- function (object, x, link = function(x) x, quantiles = c(0.
 #' @param quantiles The quantiles to calculate credible intervals for the posterior modes (in model space).
 #' @param pop The population to plot, defaults to last
 #' @param tol The tolerance to use for the correlation when finding equivalent features, default is 0.0000001
+#' @param x_train Training design matrix to be provided when imputations are to be made from them
 #' 
 #' @param ... Not used.
 #' @return A list containing aggregated predictions and per model predictions.
@@ -137,35 +151,29 @@ predict.gmjmcmc.2 <- function (object, x, link = function(x) x, quantiles = c(0.
 #' result <- gmjmcmc.parallel(
 #'  runs = 1,
 #'  cores = 1,
-#'  list(populations = "best", complex.measure = 2, tol = 0.0000001),
-#'  matrix(rnorm(600), 100),
+#'  x = matrix(rnorm(600), 100),
+#'  y = matrix(rnorm(100), 100),
 #'  P = 2,
-#'  gaussian.loglik,
-#'  loglik.alpha = gaussian.loglik.alpha,
-#'  c("p0", "exp_dbl")
+#'  transforms = c("p0", "exp_dbl")
 #' )
 #' preds <- predict(result, matrix(rnorm(600), 100))
 #'
 #' @export
-predict.gmjmcmc_merged <- function (object, x, link = function(x) x, quantiles = c(0.025, 0.5, 0.975), pop = NULL,tol =  0.0000001, ...) {
-  if(!is.null(attr(object,which = "imputed")))
-  {
-    df <- data.frame(x)
-    na.matr <- data.frame(1*(is.na(df)))
-    cm <- colMeans(na.matr)
-    na.matr <- na.matr[,attr(object,which = "imputed")]
-    names(na.matr) <- paste0("mis_",names(na.matr))
-    for (i in which(cm!=0)){
-      df[[i]][is.na(df[[i]])] <- median(df[[i]], na.rm = TRUE)
-    }
-    x <- as.matrix(data.frame(df,na.matr))
-    rm(df)
-    rm(na.matr)
-  } else x <- as.matrix(x)
+predict.gmjmcmc_merged <- function (object, x, link = function(x) x, quantiles = c(0.025, 0.5, 0.975), pop = NULL, tol = 0.0000001, x_train = NULL, ...) {
+  
+  
+  if(is.null(x_train))
+    x <- impute_x(object, x)
+  else
+    x <- impute_x_pred(object, x, x_train)
+  if (object$intercept) {
+    x <- cbind(1, x)
+  }
+  
   
   transforms.bak <- set.transforms(object$transforms)
-  if(!is.null(pop))
-    object <- merge_results(object$results.raw, pop, 2, tol, data = x)
+  if (!is.null(pop))
+    object <- merge_results(object$results.raw, pop, 2, tol, data = list(x = x, fixed = object$fixed))
   
   preds <- list()
   for (i in seq_along(object$results)) {
@@ -175,24 +183,24 @@ predict.gmjmcmc_merged <- function (object, x, link = function(x) x, quantiles =
       models <- object$results[[i]]$models[[j]]
       features <- object$results[[i]]$populations[[j]]
       model.probs <- object$results[[i]]$model.probs[[j]]
-
-      # Precalculate the features for the new data (c(0,1...) is because precalc features thinks there is an intercept and y col).
-      x.precalc <- precalc.features(cbind(0, 1, x), features)[, -1]
-
-      yhat <- matrix(0, nrow=nrow(x), ncol=length(models))
+      
+      # Precalculate the features for the new data
+      x.precalc <- precalc.features(list(x = x, fixed = object$fixed), features)$x
+      
+      yhat <- matrix(0, nrow = nrow(x), ncol = length(models))
       for (k in seq_along(models)) {
         # Models which have 0 weight are skipped since they may also be invalid, and would not influence the predictions.
         if (models[[k]]$crit == -.Machine$double.xmax) next
-        yhat[, k] <- link(x.precalc[, c(TRUE, models[[k]]$model), drop=FALSE] %*% models[[k]]$coefs)
+        yhat[, k] <- link(x.precalc[, c(rep(TRUE, object$fixed), models[[k]]$model), drop=FALSE] %*% models[[k]]$coefs)
       }
-
+      
       mean.pred <- rowSums(yhat %*% diag(as.numeric(model.probs)))
       pred.quant <- apply(yhat, 1, weighted.quantiles, weights=model.probs, prob=quantiles)
-
+      
       preds[[i]][[j]] <- list(mean=mean.pred, quantiles=pred.quant, weight=object$results[[i]]$pop.weights[j])
     }
   }
-
+  
   aggr <- list()
   aggr$mean <- 0 * preds[[1]][[1]]$mean
   aggr$quantiles <- 0 * preds[[1]][[1]]$quantiles
@@ -214,43 +222,38 @@ predict.gmjmcmc_merged <- function (object, x, link = function(x) x, quantiles =
 #' \item{quantiles}{Quantiles of aggregated predictions.}
 #' 
 #' @examples
-#' result <- mjmcmc(matrix(rnorm(600), 100), gaussian.loglik)
-#' preds <- predict(result, matrix(rnorm(500), 100))
+#' result <- mjmcmc(
+#' x = matrix(rnorm(600), 100),
+#' y = matrix(rnorm(100), 100),
+#' loglik.pi = gaussian.loglik)
+#' preds <- predict(result, matrix(rnorm(600), 100))
 #' 
 #' @export
-predict.mjmcmc <- function (object, x, link = function(x) x, quantiles = c(0.025, 0.5, 0.975), ...) {
+predict.mjmcmc <- function (object, x, link = function(x) x, quantiles = c(0.025, 0.5, 0.975), x_train = NULL, ...) {
   # Select the models and features to predict from at this iteration
-  
-  if(!is.null(attr(object,which = "imputed")))
-  {
-    df <- data.frame(x)
-    na.matr <- data.frame(1*(is.na(df)))
-    cm <- colMeans(na.matr)
-    na.matr <- na.matr[,attr(object,which = "imputed")]
-    names(na.matr) <- paste0("mis_",names(na.matr))
-    for (i in which(cm!=0)){
-      df[[i]][is.na(df[[i]])] <- median(df[[i]], na.rm = TRUE)
-    }
-    x <- as.matrix(data.frame(df,na.matr))
-    rm(df)
-    rm(na.matr)
-  } else x <- as.matrix(x)
-  
-  x <- as.matrix(cbind(rep(1, dim(x)[1]), x))
+  if(is.null(x_train))
+    x <- impute_x(object, x)
+  else
+    x <- impute_x_pred(object, x, x_train)
   
   
-  models <- c(object$models, object$lo.models)[object$model.probs.idx]
-
-  yhat <- matrix(0, nrow=nrow(x), ncol=length(models))
+  if (object$intercept) {
+    x <- cbind(1, x)
+  }
+  
+  
+  models <- object$models[object$model.probs.idx]
+  
+  yhat <- matrix(0, nrow = nrow(x), ncol = length(models))
   for (k in seq_along(models)) {
     # Models which have 0 weight are skipped since they may also be invalid, and would not influence the predictions.
     if (models[[k]]$crit == -.Machine$double.xmax) next
-    yhat[, k] <- link(x[, c(TRUE, models[[k]]$model), drop=FALSE] %*% models[[k]]$coefs)
+    yhat[, k] <- link(x[, c(rep(TRUE, object$fixed), models[[k]]$model), drop=FALSE] %*% models[[k]]$coefs)
   }
-
+  
   mean.pred <- rowSums(yhat %*% diag(as.numeric(object$model.probs)))
   pred.quant <- apply(yhat, 1, weighted.quantiles, weights = object$model.probs, prob = quantiles)
-
+  
   return(list(mean = mean.pred, quantiles = pred.quant))
 }
 
@@ -262,39 +265,26 @@ predict.mjmcmc <- function (object, x, link = function(x) x, quantiles = c(0.025
 #' \item{quantiles}{Quantiles of aggregated predictions.}
 #' 
 #' @examples
-#' result <- mjmcmc.parallel(runs = 1, cores = 1, matrix(rnorm(600), 100), gaussian.loglik)
-#' preds <- predict(result, matrix(rnorm(500), 100))
+#' result <- mjmcmc.parallel(runs = 1, 
+#' cores = 1, 
+#' x = matrix(rnorm(600), 100),
+#' y = matrix(rnorm(100), 100), 
+#' loglik.pi = gaussian.loglik)
+#' preds <- predict(result, matrix(rnorm(600), 100))
 #' 
 #' @export
-predict.mjmcmc_parallel <- function (object, x, link = function(x) x, quantiles = c(0.025, 0.5, 0.975), ...) {
-  max.crits <- numeric()
+predict.mjmcmc_parallel <- function (object, x, link = function(x) x, quantiles = c(0.025, 0.5, 0.975), x_train = NULL, ...) {
+  if(is.null(x_train))
+    x <- impute_x(object, x)
+  else
+    x <- impute_x_pred(object, x, x_train)
   
-  if(!is.null(attr(object,which = "imputed")))
-  {
-    df <- data.frame(x)
-    na.matr <- data.frame(1*(is.na(df)))
-    cm <- colMeans(na.matr)
-    na.matr <- na.matr[,attr(object,which = "imputed")]
-    names(na.matr) <- paste0("mis_",names(na.matr))
-    for (i in which(cm!=0)){
-      df[[i]][is.na(df[[i]])] <- median(df[[i]], na.rm = TRUE)
-    }
-    x <- as.matrix(data.frame(df,na.matr))
-    rm(df)
-    rm(na.matr)
-  } else x <- as.matrix(x)
-  
-  for (i in seq_along(object)) {
-    max.crits <- c(max.crits, object[[i]]$best.crit)
-  }
+  max.crits <- sapply(object$chains, function (x) x$best.crit)
   max.crit <- max(max.crits)
   result.weights <- exp(max.crits - max.crit) / sum(exp(max.crits - max.crit))
-
-  preds <- list()
-  for (i in seq_along(object)) {
-    preds[[i]] <- predict.mjmcmc(object[[i]], x, link, quantiles)
-  }
-
+  
+  preds <- lapply(object$chains, predict.mjmcmc,x, link, quantiles)
+  
   aggr <- list()
   aggr$mean <- 0 * preds[[1]]$mean
   aggr$quantiles <- 0 * preds[[1]]$quantiles
@@ -302,7 +292,7 @@ predict.mjmcmc_parallel <- function (object, x, link = function(x) x, quantiles
     aggr$mean <- aggr$mean + preds[[i]]$mean * result.weights[i]
     aggr$quantiles <- aggr$quantiles + preds[[i]]$quantiles * result.weights[i]
   }
-
+  
   return(list(aggr = aggr, preds = preds))
 }
 
@@ -318,34 +308,20 @@ predict.mjmcmc_parallel <- function (object, x, link = function(x) x, quantiles
 #' result <- gmjmcmc.parallel(
 #'  runs = 1,
 #'  cores = 1,
-#'  list(populations = "best", complex.measure = 2, tol = 0.0000001),
-#'  matrix(rnorm(600), 100),
+#'  x = matrix(rnorm(600), 100),
+#'  y = matrix(rnorm(100), 100),
 #'  P = 2,
-#'  gaussian.loglik,
-#'  loglik.alpha = gaussian.loglik.alpha,
-#'  c("p0", "exp_dbl")
+#'  transforms = c("p0", "exp_dbl")
 #' )
-#' preds <- predict(result$results, matrix(rnorm(600), 100))
+#' preds <- predict(result, matrix(rnorm(600), 100))
 #' 
 #' @export
-predict.gmjmcmc_parallel <- function (object, x, link = function(x) x, quantiles = c(0.025, 0.5, 0.975), ...) {
+predict.gmjmcmc_parallel <- function (object, x, link = function(x) x, quantiles = c(0.025, 0.5, 0.975), x_train = NULL, ...) {
   transforms.bak <- set.transforms(object$transforms)
-  
-  if(!is.null(attr(object,which = "imputed")))
-  {
-    df <- data.frame(x)
-    na.matr <- data.frame(1*(is.na(df)))
-    cm <- colMeans(na.matr)
-    na.matr <- na.matr[,attr(object,which = "imputed")]
-    names(na.matr) <- paste0("mis_",names(na.matr))
-    for (i in which(cm!=0)){
-      df[[i]][is.na(df[[i]])] <- median(df[[i]], na.rm = TRUE)
-    }
-    x <- as.matrix(data.frame(df,na.matr))
-    rm(df)
-    rm(na.matr)
-  } else x <- as.matrix(x)
-  
+  if(is.null(x_train))
+    x <- impute_x(object, x)
+  else
+    x <- impute_x_pred(object, x, x_train)
   merged <- merge_results(object,data = cbind(1, x), ...)
   results <- predict.gmjmcmc_merged(merged, x, link, quantiles)
   set.transforms(transforms.bak)
@@ -363,10 +339,51 @@ predict.gmjmcmc_parallel <- function (object, x, link = function(x) x, quantiles
 weighted.quantiles <- function (values, weights, prob = c(0.025, 0.975)) {
   ordered <- order(values)
   P <- cumsum(weights[ordered])
-
+  
   iv <- integer(length(prob))
   for (i in seq_along(iv)) {
     iv[i] <- which.max(P >= prob[i])
   }
   {values[ordered]}[iv]
 }
+
+impute_x <- function (object, x) {
+  if (!is.null(attr(object, which = "imputed"))) {
+    df <- data.frame(x)
+    na.matr <- data.frame(1 * (is.na(df)))
+    cm <- colMeans(na.matr)
+    na.matr <- na.matr[, attr(object, which = "imputed")]
+    names(na.matr) <- paste0("mis_", names(na.matr))
+    for (i in which(cm != 0)){
+      med <- median(df[[i]], na.rm = TRUE)
+      if(is.na(med))
+        stop("No data for imputation in test set, provide x_train in predict!")
+      df[[i]][is.na(df[[i]])] <- med
+    }
+    return(as.matrix(data.frame(df,na.matr)))
+  }
+  return(as.matrix(x))
+}
+
+
+impute_x_pred <- function (object, x_test, x_train) {
+  if (!is.null(attr(object, which = "imputed"))) {
+    df <- data.frame(x_test)
+    x_train <- data.frame(x_train)
+    na.matr <- data.frame(1 * (is.na(df)))
+    cm <- colMeans(na.matr)
+    na.matr <- na.matr[, attr(object, which = "imputed")]
+    names(na.matr) <- paste0("mis_", names(na.matr))
+    for (i in which(cm != 0)){
+      med <- median(x_train[[i]], na.rm = TRUE)
+      if(is.na(med))
+      {
+        warning("One or more missing in test columns do not have any data in x_train, test set will be used for imputations!")
+        med <-  median(df[[i]], na.rm = TRUE)
+      }
+      df[[i]][is.na(df[[i]])] <- med
+    }
+    return(as.matrix(data.frame(df,na.matr)))
+  }
+  return(as.matrix(x_test))
+}
